#!/usr/bin/env python3
# Copyright (C) 2024 Spearhead Systems SRL

import copy
from cmk.base.plugins.agent_based.agent_based_api.v1 import State
from cmk.gui.i18n import _
from cmk.gui.plugins.wato.utils import (
    rulespec_registry,
    HostRulespec,
    IndividualOrStoredPassword,
    RulespecGroupCheckParametersDiscovery,
    CheckParameterRulespecWithItem,
    RulespecGroupCheckParametersApplications,
)
from cmk.gui.watolib.rulespecs import Rulespec
from cmk.gui.valuespec import (
    Dictionary,
    TextInput,
    Integer,
    ListOfStrings,
    Password
)

def _discovery(title):
    return Dictionary(
        title=_(title),
        required_keys=["tenant", "username", "password"],
        elements=[
            (
                "tenant",
                TextInput(
                    title=_("Tenant ID / Directory ID"),
                    allow_empty=False,
                ),
            ),
            (
                "username",
                TextInput(
                    title=_("Client ID / Application ID"),
                    allow_empty=False,
                ),
            ),
            (
                "password",
                IndividualOrStoredPassword(
#                Password(
                    title=_("Client Secret"),
                    allow_empty=False,
                ),
            ),
            (
                "proxy",
                TextInput(
                    title=_("Proxy"),
                ),
            ),

        ],
    )

def _valuespec_special_agents_azure_keyvault_discovery():
    return _discovery("Azure Key Vault Metrics Discovery")

def _valuespec_special_agents_azure_firewall_discovery():
    return _discovery("Azure Firewall Metrics Discovery")

def _valuespec_special_agents_azure_defender_discovery():
    return _discovery("Azure Defender Alerts Discovery")

def _valuespec_special_agents_azure_keyvault_check():
    return Dictionary(
        title=_("Azure Key Vault Metric Checks"),
        elements=[
            (
                "availability",
                Tuple(
                    title=_("Availability"),
                    help=_("If drops below these percentages over the past minute, issue alert"),
                    elements=[
                        Percentage(
                            title=_("Warn if below"),
                            default_value=98
                        ),
                        Percentage(
                            title=_("Crit if below"),
                            default_value=90
                        )
                    ]
                )
            ),
            (
                "capacity",
                Tuple(
                    title=_("Capacity used"),
                    help=_("If goes above these percentages over the past minute, issue alert"),
                    elements=[
                        Percentage(
                            title=_("Warn if above"),
                            default_value=80
                        ),
                        Percentage(
                            title=_("Crit if above"),
                            default_value=98
                        )
                    ]
                )
            ),
            (
                "latency",
                Tuple(
                    title=_("Request latency"),
                    help=_("If goes above the average milliseconds over the past minute, issue alert"),
                    elements=[
                        Integer(
                            title=_("Warn if above"),
                            default_value=100,
                            minvalue=0,
                        ),
                        Integer(
                            title=_("Crit if above"),
                            default_value=2000,
                            minvalue=0,
                        )
                    ]
                )
            ),
        ],
    )

def _valuespec_special_agents_azure_firewall_check():
    return Dictionary(
        title=_("Azure Firewall Metric Checks"),
        elements=[
            (
                "availability",
                Tuple(
                    title=_("Availability"),
                    help=_("If drops below these percentages over the past minute, issue alert"),
                    elements=[
                        Percentage(
                            title=_("Warn if below"),
                            default_value=98
                        ),
                        Percentage(
                            title=_("Crit if below"),
                            default_value=90
                        )
                    ]
                )
            ),
            (
                "latency",
                Tuple(
                    title=_("Request latency"),
                    help=_("If goes above the average milliseconds over the past minute, issue alert"),
                    elements=[
                        Integer(
                            title=_("Warn if above"),
                            default_value=100,
                            minvalue=0,
                        ),
                        Integer(
                            title=_("Crit if above"),
                            default_value=2000,
                            minvalue=0,
                        )
                    ]
                )
),
        ],
    )

def _valuespec_special_agents_azure_defender_check():
    return Dictionary(
        title=_("Azure Defender Alerts Severity"),
        elements=[
            (
                "severity_high",
                DropdownChoice(
                    title=_("Defender severity 'High'"),
                    help=_("What CheckMK criticality should this Azure Defender severity trigger"),
                    default_value=State.CRIT.value,
                    choices=[
                        (State.CRIT.value, _(State.CRIT.name)),
                        (State.WARN.value, _(State.WARN.name)),
                        (State.OK.value,   _(State.OK.name)),
                    ],
                ),
            ),
            (
                "severity_medium",
                DropdownChoice(
                    title=_("Defender severity 'Medium'"),
                    help=_("What CheckMK criticality should this Azure Defender severity trigger"),
                    default_value=State.WARN.value,
                    choices=[
                        (State.CRIT.value, _(State.CRIT.name)),
                        (State.WARN.value, _(State.WARN.name)),
                        (State.OK.value,   _(State.OK.name)),
                    ],
                ),
            ),
            (
                "severity_low",
                DropdownChoice(
                    title=_("Defender severity 'Low'"),
                    help=_("What CheckMK criticality should this Azure Defender severity trigger"),
                    default_value=State.WARN.value,
                    choices=[
                        (State.CRIT.value, _(State.CRIT.name)),
                        (State.WARN.value, _(State.WARN.name)),
                        (State.OK.value,   _(State.OK.name)),
                    ],
                ),
            ),
            (
                "severity_informational",
                DropdownChoice(
                    title=_("Defender severity 'Informational'"),
                    help=_("What CheckMK criticality should this Azure Defender severity trigger"),
                    default_value=State.OK.value,
                    choices=[
                        (State.CRIT.value, _(State.CRIT.name)),
                        (State.WARN.value, _(State.WARN.name)),
                        (State.OK.value,   _(State.OK.name)),
                    ],
                ),
            ),
        ],
    )

rulespec_registry.register(
    HostRulespec(
        name="special_agents:azure_keyvault",
        group=RulespecGroupCheckParametersDiscovery,
        match_type='dict',
        valuespec=_valuespec_special_agents_azure_keyvault_discovery,
    )
)
rulespec_registry.register(
    HostRulespec(
        name="special_agents:azure_firewall",
        group=RulespecGroupCheckParametersDiscovery,
        match_type='dict',
        valuespec=_valuespec_special_agents_azure_firewall_discovery,
    )
)
rulespec_registry.register(
    HostRulespec(
        name="special_agents:azure_defender",
        group=RulespecGroupCheckParametersDiscovery,
        match_type='dict',
        valuespec=_valuespec_special_agents_azure_defender_discovery,
    )
)

rulespec_registry.register(
    CheckParameterRulespecWithItem(
        check_group_name="azure_keyvault",
        group=RulespecGroupCheckParametersApplications,
        match_type="dict",
        parameter_valuespec=_valuespec_special_agents_azure_keyvault_check,
        item_spec=lambda: TextInput(title=_("Key Vault")),
        title=lambda: _("Azure Key Vault Metrics"),
    )
)
rulespec_registry.register(
    CheckParameterRulespecWithItem(
        check_group_name="azure_firewall",
        group=RulespecGroupCheckParametersApplications,
        match_type="dict",
        parameter_valuespec=_valuespec_special_agents_azure_firewall_check,
        item_spec=lambda: TextInput(title=_("Firewall")),
        title=lambda: _("Azure Firewall Metrics"),
    )
)
rulespec_registry.register(
    CheckParameterRulespecWithItem(
        check_group_name="azure_defender",
        group=RulespecGroupCheckParametersApplications,
        match_type="dict",
        parameter_valuespec=_valuespec_special_agents_azure_defender_check,
        item_spec=lambda: TextInput(title=_("Defender")),
        title=lambda: _("Azure Defender Alerts Severity"),
    )
)
